from .common import Path
import logging
from .config_class import PhisConfig


_singleton_instance: PhisConfig | None = None


def _get_instance() -> PhisConfig:
    global _singleton_instance
    if _singleton_instance is None:
        if Path.cwd().joinpath('文档').exists():
            _singleton_instance = PhisConfig(Path.cwd())
        else:
            logging.warning('未设置医院配置目录，使用当前工作目录')
            _singleton_instance = PhisConfig(Path.cwd())
    return _singleton_instance


class ProgramConfigV2:
    @classmethod
    def set_hospital_config_dir(cls, config_dir: Path):
        global _singleton_instance
        _singleton_instance = PhisConfig(config_dir)

    @classmethod
    def get_config(cls, file_relative_path: str):
        return _get_instance().get_config(file_relative_path)

    @classmethod
    def get_url(cls):
        return _get_instance().get_url()

    @classmethod
    def get_username(cls):
        return _get_instance().get_username()

    @classmethod
    def get_password(cls):
        return _get_instance().get_password()

    @classmethod
    def get_department_name(cls):
        return _get_instance().get_department_name()

    @classmethod
    def get_follow_up_start_date(cls):
        return _get_instance().get_follow_up_start_date()

    @classmethod
    def get_follow_up_end_date(cls):
        return _get_instance().get_follow_up_end_date()

    @classmethod
    def get_completed_count(cls):
        return _get_instance().get_completed_count()

    @classmethod
    def get_organization_name(cls):
        return _get_instance().get_organization_name()

    @classmethod
    def use_other_doctor_records(cls) -> bool:
        return _get_instance().use_other_doctor_records()

    @classmethod
    def continue_save_if_already_done_test(cls) -> bool:
        return _get_instance().continue_save_if_already_done_test()

    @classmethod
    def no_diabetes_record_fasting_blood_sugar(cls) -> bool:
        return _get_instance().no_diabetes_record_fasting_blood_sugar()

    @classmethod
    def introduction_medication_start_date(cls):
        return _get_instance().introduction_medication_start_date()

    @classmethod
    def introduction_medication_end_date(cls):
        return _get_instance().introduction_medication_end_date()



"""
程序配置文件解析
"""

from pathlib import Path
from .common import get_line_option, get_raw_string


class ProgramConfig:
    """
    程序配置文件解析
    """

    def __init__(self, path):
        self.env_file = Path(path)

    @property
    def 已完成数量(self):
        return get_line_option(self.env_file, 3)

    @property
    def 机构名称(self):
        return get_line_option(self.env_file, 5)

    @property
    def continue_save_if_already_done_test(self):
        "本季度已做过慢病随访，是否继续保存"
        return get_line_option(self.env_file, 6)

    @property
    def 无糖尿病是否录入空腹血糖(self):
        return get_line_option(self.env_file, 7)

    @property
    def 引入用药起始时间(self):
        return get_line_option(self.env_file, 8)

    @property
    def 引入用药结束时间(self):
        return get_line_option(self.env_file, 9)


class AdminConfig1:
    def __init__(self, path):
        self.env_file = Path(path)

    @property
    def 随访新建起始时间(self):
        return get_line_option(self.env_file, 5)

    @property
    def 随访新建结束时间(self):
        return get_line_option(self.env_file, 6)

    @property
    def 登录网址(self):
        return get_raw_string(self.env_file, 1)

    @property
    def 登录用户名(self):
        return get_raw_string(self.env_file, 2)

    @property
    def 登录密码(self):
        return get_raw_string(self.env_file, 3)

    @property
    def 登录科室名称(self):
        return get_raw_string(self.env_file, 4)


Config = ProgramConfig('执行结果/env.txt')
"""执行结果配置"""

AdminConfig = AdminConfig1('文档/admin.txt')
"""Admin配置 （随访新建起始时间，随访新建结束时间）"""



from .common import (
    get_raw_string,
    get_line_option,
    Path,
    get_line_option_as_bool,
    get_line_option_as_int,
)
import logging


class PhisConfig:
    def __init__(self, hospital_config_dir: Path):
        if not hospital_config_dir.exists():
            logging.warning(f'医院配置目录 {hospital_config_dir} 不存在，使用当前目录 {Path.cwd()}')
            self.hospital_config_dir = Path.cwd()
        else:
            self.hospital_config_dir = hospital_config_dir

    def get_config(self, file_relative_path: str) -> Path:
        return self.hospital_config_dir.joinpath(file_relative_path)

    def get_url(self) -> str:
        return get_raw_string(self.get_config('文档/admin.txt'), 1)

    def get_username(self) -> str:
        return get_raw_string(self.get_config('文档/admin.txt'), 2)

    def get_password(self) -> str:
        return get_raw_string(self.get_config('文档/admin.txt'), 3)

    def get_department_name(self) -> str:
        return get_raw_string(self.get_config('文档/admin.txt'), 4)

    def get_follow_up_start_date(self) -> str:
        return get_line_option(self.get_config('文档/admin.txt'), '随访新建起始时间')

    def get_follow_up_end_date(self) -> str:
        return get_line_option(self.get_config('文档/admin.txt'), '随访新建结束时间')

    def get_completed_count(self) -> int:
        return get_line_option_as_int(self.get_config('执行结果/env.txt'), '已完成数量')

    def get_organization_name(self) -> str:
        return get_line_option(self.get_config('执行结果/env.txt'), '机构名称')

    def use_other_doctor_records(self) -> bool:
        # 没有签约医生的门诊记录, 是否需要判别包含机构名称字样的其他医生的门诊记录
        return get_line_option_as_bool(
            self.get_config('执行结果/env.txt'), '没有签约医生的门诊'
        )

    def continue_save_if_already_done_test(self) -> bool:
        # 本季度已做过慢病随访，是否继续保存
        return get_line_option_as_bool(
            self.get_config('执行结果/env.txt'), '本季度已做过慢病随访，是否继续保存'
        )

    def no_diabetes_record_fasting_blood_sugar(self) -> bool:
        # 无糖尿病是否录入空腹血糖
        return get_line_option_as_bool(
            self.get_config('执行结果/env.txt'), '无糖尿病是否录入空腹血糖'
        )

    def introduction_medication_start_date(self) -> str:
        # 引入用药起始时间
        return get_line_option(self.get_config('执行结果/env.txt'), '引入用药起始时间')

    def introduction_medication_end_date(self) -> str:
        # 引入用药结束时间
        return get_line_option(self.get_config('执行结果/env.txt'), '引入用药结束时间')


from phis_config.config import Config, AdminConfig
from .common import 重置已完成数量
from .config_v2 import ProgramConfigV2
from .config_class import PhisConfig

__all__ = ['Config', 'AdminConfig', '重置已完成数量', 'ProgramConfigV2', 'PhisConfig']



from pathlib import Path
import logging
from typing import Union


def check_exists(file: Union[str, Path]) -> None:
    file_path = Path(file).resolve()
    if not file_path.exists():
        logging.fatal(f'文件不存在，请检查文件路径是否正确. {file_path}')
        exit(-1)


def env_write(file_path, line_number, content):
    # 读取文件并将其内容存储到列表中
    check_exists(file_path)
    with open(file_path, 'r', encoding='utf-8') as file:
        lines = file.readlines()

    # 修改指定行的内容（注意列表是从0开始索引的）
    if 1 <= line_number <= len(lines):
        lines[line_number - 1] = content + '\n'
    elif line_number > len(lines):
        # 如果指定的行数超出了文件的总行数，将在末尾添加新行
        lines.extend(['\n'] * (line_number - len(lines) - 1))
        lines.append(content + '\n')

    # 将修改后的内容写回文件
    with open(file_path, 'w', encoding='utf-8') as file:
        file.writelines(lines)


def get_line_option(
    env_file: Path, line_number_or_start_string: Union[int, str]
) -> str:
    """
    获取配置文件中第N行数据的值 (格式: KEY:VALUE)
    或者：获取配置文件中以XXX开头的行的值
    """
    check_exists(env_file)
    if isinstance(line_number_or_start_string, int):
        config_content = env_file.read_text(encoding='utf8').split('\n')
        line_number_in_file = line_number_or_start_string - 1  # 实际行数从0开始
        if line_number_in_file < 0 or line_number_in_file >= len(config_content):
            logging.fatal('配置文件行数错误！', line_number_or_start_string)
        return (
            config_content[line_number_in_file].replace('：', ':').split(':')[1].strip()
        )
    elif isinstance(line_number_or_start_string, str):
        config_content = env_file.read_text(encoding='utf8').split('\n')
        for line in config_content:
            if line.startswith(line_number_or_start_string):
                return line.replace('：', ':').split(':')[1].strip()
        logging.fatal(f'配置文件中没有找到以"{line_number_or_start_string}"开头的行')

    else:
        logging.error('line_number_or_start_string 必须是 int 或 str 类型')


def get_line_option_as_int(
    env_file: Path, line_number_or_start_string: Union[int, str]
) -> int:
    """
    获取配置文件中第N行数据的值 (格式: KEY:VALUE) 并转换为整数
    """
    value = get_line_option(env_file, line_number_or_start_string)
    try:
        return int(value)
    except ValueError:
        logging.fatal(f'无法将 "{value}" 转换为整数')
        return 0  # 或者抛出异常，根据需要处理


def get_line_option_as_bool(
    env_file: Path, line_number_or_start_string: Union[int, str]
) -> bool:
    """
    获取配置文件中第N行数据的值 (格式: KEY:VALUE) 并转换为布尔值
    """
    value = get_line_option(env_file, line_number_or_start_string)
    if value.lower() in ['是', 'true', '1']:
        return True
    elif value.lower() in ['否', 'false', '0']:
        return False
    else:
        logging.fatal(f'无法将 "{value}" 转换为布尔值')
        return False  # 或者抛出异常，根据需要处理


def get_raw_string(env_file, line_number: int) -> str:
    "获取配置文件中第N行数据, 原字符串"
    check_exists(env_file)
    config_content = env_file.read_text(encoding='utf8').split('\n')
    line_number_in_file = line_number - 1  # 实际行数从0开始
    return config_content[line_number_in_file].strip()


def 重置已完成数量():
    env_write('执行结果/env.txt', 3, f'已完成数量:0')
